package sun.security.x509;

import java.util.Enumeration;
import sun.security.util.DerOutputStream;
import java.io.OutputStream;
import sun.security.util.DerValue;
import java.io.InputStream;
import java.io.IOException;
import sun.security.util.DerInputStream;
import javax.security.auth.x500.X500Principal;

public class CertificateIssuerName implements CertAttrSet<String>
{
    public static final String IDENT = "x509.info.issuer";
    public static final String NAME = "issuer";
    public static final String DN_NAME = "dname";
    public static final String DN_PRINCIPAL = "x500principal";
    private X500Name dnName;
    private X500Principal dnPrincipal;
    
    public CertificateIssuerName(X500Name dnName) {
        this.dnName = dnName;
    }
    
    public CertificateIssuerName(DerInputStream derInputStream) throws IOException {
        this.dnName = new X500Name(derInputStream);
    }
    
    public CertificateIssuerName(InputStream in) throws IOException {
        this.dnName = new X500Name(new DerValue(in));
    }
    
    @Override
    public String toString() {
        if (this.dnName == null) {
            return "";
        }
        return this.dnName.toString();
    }
    
    @Override
    public void encode(OutputStream outputStream) throws IOException {
        DerOutputStream derOutputStream = new DerOutputStream();
        this.dnName.encode(derOutputStream);
        outputStream.write(derOutputStream.toByteArray());
    }
    
    @Override
    public void set(String s, Object o) throws IOException {
        if (!(o instanceof X500Name)) {
            throw new IOException("Attribute must be of type X500Name.");
        }
        if (s.equalsIgnoreCase("dname")) {
            this.dnName = (X500Name)o;
            this.dnPrincipal = null;
            return;
        }
        throw new IOException("Attribute name not recognized by CertAttrSet:CertificateIssuerName.");
    }
    
    @Override
    public Object get(String s) throws IOException {
        if (s.equalsIgnoreCase("dname")) {
            return this.dnName;
        }
        if (s.equalsIgnoreCase("x500principal")) {
            if (this.dnPrincipal == null && this.dnName != null) {
                this.dnPrincipal = this.dnName.asX500Principal();
            }
            return this.dnPrincipal;
        }
        else {
            throw new IOException("Attribute name not recognized by CertAttrSet:CertificateIssuerName.");
        }
    }
    
    @Override
    public void delete(String s) throws IOException {
        if (s.equalsIgnoreCase("dname")) {
            this.dnName = null;
            this.dnPrincipal = null;
            return;
        }
        throw new IOException("Attribute name not recognized by CertAttrSet:CertificateIssuerName.");
    }
    
    @Override
    public Enumeration<String> getElements() {
        AttributeNameEnumeration attributeNameEnumeration = new AttributeNameEnumeration();
        attributeNameEnumeration.addElement("dname");
        return attributeNameEnumeration.elements();
    }
    
    @Override
    public String getName() {
        return "issuer";
    }
}
